
{*******************************************************}
{                                                       }
{       RVLister                                        }
{       DLL - plugin for Total Commander.               }
{       Viewing and printing RVF files.                 }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{       Total Commander (c) Christian Ghisler           }
{       http://www.ghisler.com                          }
{                                                       }
{*******************************************************}

{
  History:
  2015-May-04: Unicode support, 64-bit support
  2008-Feb-19: Support for gif animation
  2004-Jan-01: Initial release
}

library RVLister;

uses
  SysUtils,
  Windows,
  Messages,
  Classes,
  Controls,
  RVStyle,
  RVScroll,
  RichView,
  StdCtrls,
  Printers,
  PtblRV,
  Dialogs,
  GifImg,
  PngImage,
  CRVFData,
  ShellApi,
  listplug in 'listplug.pas',
  RVGifAnimate2007;

{
  This code uses the following graphic classes:

  GifImage (by Anders Melander)
    http://www.torry.net/vcl/graphics/gif/gifimage.exe (original)
    http://clootie.narod.ru/delphi/download_vcl.html (update)
  PngObject (by Gustavo Huffenbacher Daud)
    http://pngdelphi.sourceforge.net/

  Important: all code in GifImage related to DummyThread must be commented out
  in order to use it in this DLL!!!
}

{$R *.res}

const RVFExt = 'RVF';

// Fake object for event processing
type
  TFakeObject = class
    procedure DoJump(Sender: TObject; id: Integer);
    procedure DoVScrolled(Sender: TObject);
  end;

procedure TFakeObject.DoJump(Sender: TObject; id: Integer);
var RVData: TCustomRVFormattedData;
    ItemNo: Integer;
begin
  TRichView(Sender).GetJumpPointLocation(id, RVData, ItemNo);
  ShellExecute(0, 'open', PChar(RVData.GetItemTag(ItemNo)), nil, nil, SW_SHOW);
end;

procedure TFakeObject.DoVScrolled(Sender: TObject);
var rv: TRichView;
    pos: Integer;
begin
  rv := TRichView(Sender);
  pos := rv.VScrollPos*rv.VSmallStep+rv.ClientHeight;
  pos := MulDiv(pos,100,rv.DocumentHeight);
  if pos>100 then
    pos := 100;
  PostMessage(rv.ParentWindow,WM_COMMAND, MAKELONG(pos,itm_percent), rv.Handle);
end;

var obj: TFakeObject = nil;
{------------------------------------------------------------------------------}
// Returning string which will be used by TC for determining if this plugin
// can display the given file
procedure ListGetDetectStringW(DetectString: PChar; maxlen: Integer); stdcall;
begin
  //MessageBox(0, 'ListGetDetectString', 'ListGetDetectString', 0);
  StrCopy(DetectString, PChar('EXT="'+RVFExt+'" & [0]="-" & [1]="8" & [2]=" " & [3]="1"'));
end;
{------------------------------------------------------------------------------}
// Creating viewer control
function ListLoadW(ParentWin:THandle; FileToLoad:PChar;
  ShowFlags:Integer):THandle; stdcall;
var rv: TRichView;
begin
  //MessageBox(0, 'ListLoad', 'ListLoad', 0);
  Result := 0;
  if AnsiCompareText(ExtractFileExt(FileToLoad),'.'+RVFExt)<>0 then
   exit;
  rv := TRichView.Create(nil);
  rv.ParentWindow := ParentWin;
  Result := rv.Handle;
  rv.Style := TRVStyle.Create(rv);
  rv.Options := rv.Options;
  rv.AnimationMode := rvaniOnFormat;
  if (ShowFlags and lcp_wraptext)<>0 then
    rv.Options := rv.Options + [rvoClientTextWidth];
  {
  // if you want to load RTF files...
  rv.RTFReadProperties.UnicodeMode := rvruMixed;
  rv.RTFReadProperties.TextStyleMode := rvrsAddIfNeeded;
  rv.RTFReadProperties.ParaStyleMode := rvrsAddIfNeeded;
  }
  rv.RVFTextStylesReadMode := rvf_sInsertMerge;
  rv.RVFParaStylesReadMode := rvf_sInsertMerge;
  rv.RVFOptions := rv.RVFOptions + [rvfoIgnoreUnknownPicFmt,
    rvfoIgnoreUnknownCtrls, rvfoConvUnknownStylesToZero,
    rvfoConvLargeImageIdxToZero, rvfoLoadBack, rvfoLoadLayout];
  rv.OnJump := obj.DoJump;
  rv.OnVScrolled := obj.DoVScrolled;
  rv.OnResize := obj.DoVScrolled;
  rv.LoadRVF(FileToLoad);
  rv.Format;
end;
{------------------------------------------------------------------------------}
// Destroying viewer control
procedure ListCloseWindow(ListWin: THandle); stdcall;
begin
  //MessageBox(0, 'ListCloseWindow', 'ListCloseWindow', 0);
  FindControl(ListWin).Free;
end;
{------------------------------------------------------------------------------}
// Processing commands from TC
function ListSendCommand(ListWin: THandle;
  Command,Parameter: Integer):Integer; stdcall;
var rv: TRichView;
begin
  //MessageBox(0, 'ListSendCommand', 'ListSendCommand', 0);
  Result := LISTPLUGIN_ERROR;
  rv := TRichView(FindControl(ListWin));
  if rv=nil then begin
    exit;
  end;
  case Command of
    lc_copy:
      rv.CopyDef;
    lc_selectall:
      begin
        rv.SelectAll;
        rv.Invalidate;
      end;
    lc_setpercent:
      begin
        Parameter := MulDiv(Parameter, rv.DocumentHeight, 100);
        dec(Parameter, rv.ClientHeight);
        rv.ScrollTo(Parameter);
      end;
    lc_newparams:
      begin
        if ((lcp_wraptext and Parameter)<>0) and
           not (rvoClientTextWidth in rv.Options) then begin
          rv.Options := rv.Options + [rvoClientTextWidth];
          rv.Format;
          end
        else if ((lcp_wraptext and Parameter)=0) and
          (rvoClientTextWidth in rv.Options) then begin
          rv.Options := rv.Options - [rvoClientTextWidth];
          rv.Format;
        end;
      end;
  end;
  Result := LISTPLUGIN_OK;
end;
{------------------------------------------------------------------------------}
// Searching text
function ListSearchTextW(ListWin: THandle; SearchString: PChar;
  SearchParameter:integer):integer; stdcall;
var Options: TRVSearchOptions;
  rv: TRichView;
begin
  Result := LISTPLUGIN_ERROR;
  rv := TRichView(FindControl(ListWin));
  if rv=nil then begin
    exit;
  end;
  if (SearchParameter and lcs_matchcase)<>0 then
    Include(Options, rvsroMatchCase);
  if (SearchParameter and lcs_wholewords)<>0 then
    Include(Options, rvsroWholeWord);
  if (SearchParameter and lcs_backwards)=0 then
    Include(Options, rvsroDown);
  if rv.SearchText(SearchString, Options) then
    Result := LISTPLUGIN_OK
  else
    Result := LISTPLUGIN_ERROR;
end;
{------------------------------------------------------------------------------}
{ Printing }
function ListPrintW(ListWin:thandle;FileToPrint,DefPrinter:pchar;
  PrintFlags:integer;var Margins:trect):integer; stdcall;
var
  rv: TRichView;
  rvp: TRVPrint;
  pd: TPrintDialog;
begin
  Result := LISTPLUGIN_ERROR;
  rv := TRichView(FindControl(ListWin));
  if (rv=nil) or (rv.ItemCount=0) then
    exit;
  if DefPrinter=nil then
    Printer.PrinterIndex := -1
  else
    Printer.PrinterIndex := Printer.Printers.IndexOf(DefPrinter);
  rvp := TRVPrint.Create(rv);
  try
    rvp.LeftMarginMM   := Round(Margins.Left/10);
    rvp.RightMarginMM  := Round(Margins.Right/10);
    rvp.TopMarginMM    := Round(Margins.Top/10);
    rvp.BottomMarginMM := Round(Margins.Bottom/10);
    rvp.AssignSource(rv);
    rvp.FormatPages(rvdoALL);
    pd := TPrintDialog.Create(nil);
    try
      pd.Options := [poPageNums];
      pd.MinPage := 1;
      pd.MaxPage := rvp.PagesCount;
      pd.FromPage := 1;
      pd.ToPage := rvp.PagesCount;
      if pd.Execute then
        case pd.PrintRange of
          prAllPages:
            rvp.Print(ExtractFileName(FileToPrint), pd.Copies, False);
          prPageNums:
            rvp.PrintPages(pd.FromPage, pd.ToPage, ExtractFileName(FileToPrint),
              pd.Copies, False);
        end;
    finally
      pd.Free;
    end;
  finally
    rvp.Free;
  end;
  Result := LISTPLUGIN_OK
end;
{------------------------------------------------------------------------------}
exports
  ListLoadW, ListCloseWindow, ListGetDetectStringW, ListSendCommand,
  ListSearchTextW, ListPrintW;
{------------------------------------------------------------------------------}

begin
  RegisterClasses([TButton, TEdit, TCheckBox, TListBox, TComboBox, TMemo, TLabel]);

end.
